<xsl:stylesheet version="1.0"
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns="http://www.w3.org/1999/xhtml">

<!--
 
 Vulnerability Assessment Report (HTML)
 Copyright (c) 2003 Star Chamber Inc.
 
 Based on nessus.xsl by Axel Nennker <axel@nennker.de>

-->

<xsl:output
	method="html" indent="yes" encoding="iso-8859-1"
	doctype-public="-//W3C//DTD XHTML 1.0 Strict//EN"
	doctype-system="http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd"/>
<xsl:strip-space elements="*"/>

<!-- ============================ Report Style Sheet =============================== -->

<xsl:template name="sussen-report-style">
	<style type="text/css">
		<xsl:comment>

		body { 
			background-color: #ffffff; 
   			font: 10pt Verdana, Geneva, Arial, Helvetica, sans-serif;
			margin:10px 10px 0px 10px;
			padding:0px;
		}

		p {
			font: 10pt Verdana, Geneva, Arial, Helvetica, sans-serif;
			margin-top: 0px;
			text-align: justify;
		}

		h2 { 
			background: #99cccc;
			color: Black;
		}

		#container {
			margin: 0px;
			padding: 0px 175px 0px 110px
		}

		#intro {
			min-width: 470px;
		}
		
		#pageHeader {
			margin-bottom: 20px; 
		}
	
		#summary {
			margin: 20px 20px 20px 10px; 
		}

		#leftcontent {
			float: left;
			width: 67%;
			border: 1px solid #000;
			margin-right:15px;
			padding: 0px 10px 0px 10px;
		}

		#rightcontent {
			border: 1px solid #000;
			padding: 0px 10px 0px 10px;
		}

		#main_left {
			width:33%;
			float:left;
			padding-bottom:10px;
		}

		#main_center {
			width: 34%;
			float: left;
			padding-bottom:10px;
		}

		#main_right {
			width:33%;
			float:right;
			padding-bottom:10px;
		}

		#normal {
			width: 100%;
			float: left;
		}

		#footer {
			text-align: center;
		}

	</xsl:comment>
	</style>
</xsl:template>

<xsl:template match="/">
<html>
	<head>
		<title>Sussen Vulnerability Assessment Report</title>
		<xsl:call-template name="sussen-report-style"/>
	</head>
	<body>
	<div id="container">
		<xsl:apply-templates select="report"/>
	</div>
	</body>
</html>
</xsl:template>

<!-- ============================ Report Header =============================== -->

<xsl:template name="sussen-report-header">

	<div id="intro">	
		<div id="pageHeader">
			<h2><span>Vulnerability Assessment Report</span></h2>
		</div>
		
		<div id="summary">
			<p class="bodycopy">This report gives details on hosts that were tested
			and any issues that were found. Please follow the recommended
			actions to help migigate these threats.</p>
		</div>	
	</div>

</xsl:template>

<!-- ============================ Scan Details =============================== -->

<xsl:template match="results" mode="scan-details">

	<h2><span>Scan Details</span></h2>
	
	<div id="leftcontent">Hosts which where alive and responding during test:</div>
	<div id="rightcontent"><xsl:value-of select="count(./result)"/></div>
	
	<div id="leftcontent">Number of security holes found:</div> 
	<div id="rightcontent"><xsl:value-of select="count(./result/ports/port/information/severity[text()='Security Hole'])"/></div>
	
	<div id="leftcontent">Number of security warnings found:</div>
	<div id="rightcontent"><xsl:value-of select="count(./result/ports/port/information/severity[text()='Security Warning'])"/></div>

</xsl:template>

<!-- ============================ Host List =============================== -->

<xsl:template match="results" mode="toc">

	<h2><span>Host List</span></h2>
	<xsl:for-each select="./result">
		<p><xsl:apply-templates select="." mode="results-toc"/></p>
	</xsl:for-each>

</xsl:template>

<xsl:template match="result" mode="results-toc">

	<div id="leftcontent">
			<xsl:value-of select="host/@name"/>
	</div>

	<xsl:choose>
		<xsl:when test="count(./ports/port/information/severity[text()='Security Hole']) &gt; 0">
			<div id="rightcontent">Security hole(s) found</div>
		</xsl:when>

		<xsl:when test="count(./ports/port/information/severity[text()='Security Warning']) &gt; 0">
			<div id="rightcontent">Security warnings(s) found</div>
		</xsl:when>

		<xsl:when test="count(./ports/port/information/severity[text()='Security Note']) &gt; 0">
			<div id="rightcontent">Security note(s) found</div>
		</xsl:when>

		<xsl:otherwise>
			<div id="rightcontent">No noticeable information found</div>
		</xsl:otherwise>
	</xsl:choose>

</xsl:template>


<!-- ============================ Host Details =============================== -->


<xsl:template match="result" mode="toc">

	<h2><span>Host Details</span></h2>
		
	<xsl:variable name="hostname">
		<xsl:value-of select="host/@name"/>
	</xsl:variable>

	<xsl:for-each select="ports/port">
		<div id="main_left">
			<xsl:value-of select="$hostname"/>
		</div>
			
		<div id="main_center">
		<xsl:value-of select="./service/@name"/>			
		<xsl:if test="./@portid">				
				<xsl:text>(</xsl:text>
				<xsl:value-of select="./@portid"/><xsl:text>/</xsl:text>
				<xsl:value-of select="./@protocol"/><xsl:text>)</xsl:text>
		</xsl:if>
        </div>
		
		<div id="main_right">
		<xsl:choose>
			<xsl:when test="count(./information/severity[text()='Security Hole']) &gt; 0">	
				Security hole(s) found					
			</xsl:when>
			<xsl:when test="count(./information/severity[text()='Security Warning']) &gt; 0">
				Security warnings(s) found
			</xsl:when>
			<xsl:when test="count(./information/severity[text()='Security Note']) &gt; 0">
				Security note(s) found
			</xsl:when>
			<xsl:otherwise>
				No noticeable information found
			</xsl:otherwise>
		</xsl:choose>
		</div>
	</xsl:for-each>
</xsl:template>

<!-- ============================ Security Issues =============================== -->

<xsl:template match="data">
	<xsl:apply-templates select="p"/>
</xsl:template>

<xsl:template name="nessus-report-issues">

	<xsl:param name="severity_type"/>
	<xsl:param name="severity_text"/>
	<xsl:param name="fontcolor" select="'black'"/>
	<xsl:for-each select="ports/port"> 
		<xsl:if test="count(./information/severity[text()=$severity_type]) &gt; 0">
			<h3>Severity:</h3><xsl:value-of select="$severity_text"/>
			
			<h3>Service:</h3><xsl:value-of select="service/@name"/>
		
			<xsl:if test="./@portid">
				<xsl:text>(</xsl:text>
				<xsl:value-of select="./@portid"/>
				<xsl:text>/</xsl:text>
				<xsl:value-of select="./@protocol"/>
				<xsl:text>)</xsl:text>
			</xsl:if>
			
			<h3>Information</h3><xsl:value-of select="./information/data"/><hr />
		</xsl:if>
	</xsl:for-each>
</xsl:template>

<xsl:template match="result" mode="data">

	<div id="normal">
		<h2><span>Security Issues</span></h2>
 	</div>
	
	<xsl:call-template name="nessus-report-issues">
		<xsl:with-param name="severity_type" select="'Security Hole'"/>
		<xsl:with-param name="severity_text" select="'Vulnerability'"/>
		<xsl:with-param name="fontcolor" select="'red'"/>
	</xsl:call-template>

	<xsl:call-template name="nessus-report-issues">
		<xsl:with-param name="severity_type" select="'Security Warning'"/>
		<xsl:with-param name="severity_text" select="'Warning'"/>
	</xsl:call-template>
		
	<xsl:call-template name="nessus-report-issues">
		<xsl:with-param name="severity_type" select="'Security Note'"/>
		<xsl:with-param name="severity_text" select="'Informational'"/>
	</xsl:call-template>
	
</xsl:template>


<xsl:template match="p">
 <xsl:choose>
  <xsl:when test="contains(.,'CVE : ')">
    <xsl:value-of select="substring-before(.,'CVE :')"/>
 
    <xsl:variable name="after-cve">
     <xsl:value-of select="substring-after(.,'CVE : ')"/>
    </xsl:variable>
    <xsl:variable name="cve">
     <xsl:value-of select="substring($after-cve,1,13)"/>
    </xsl:variable>
 
    <a href="http://cgi.nessus.org/cve.php3?cve={$cve}">
     <xsl:text>CVE : </xsl:text>
     <xsl:value-of select="$cve"/>
    </a>
 
    <xsl:value-of select="substring($after-cve, 14)"/>
  </xsl:when>
  <xsl:otherwise>
   <xsl:value-of select="."/>
  </xsl:otherwise>
 </xsl:choose>
</xsl:template>

<xsl:template match="results">
 <xsl:apply-templates select="." mode="scan-details"/>
 <xsl:apply-templates select="." mode="toc"/>
 <xsl:apply-templates select="." mode="analysis"/>
</xsl:template>

<xsl:template match="results" mode="analysis">
	<xsl:for-each select="result">
		<xsl:apply-templates select="." mode="toc"/>
		<br/><br/>
		<xsl:apply-templates select="." mode="data"/>
	</xsl:for-each>
</xsl:template>

<xsl:template match="report">
   <xsl:call-template name="sussen-report-header"/>
   <xsl:apply-templates select="results"/>
   
   <div id="footer">
   Report generated by <a href="http://sussen.sourceforge.net/">Sussen</a>
   </div>
   
</xsl:template>


</xsl:stylesheet>
