/* 
 * Copyright (C) 2003 Star Chamber Inc.
 *
 * Sussen - sussen-auth.c
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Author: Loren Bandiera <lorenb@starchamber.ca>
 */

#include "sussen.h"
#include "sussen-auth.h"
#include "sussen-network.h"
#include "sussen-util.h"

GladeXML *xml;
GtkWidget *auth_dialog;
GtkWidget *login_button;

gchar *g_host;
gchar *g_port;

void do_server_login (GtkWidget *widget, gpointer user_data) {

	GtkWidget *w;
	gint port = (gint) atoi(g_port);
	const gchar *username, *password;

	w = glade_xml_get_widget (xml,"auth_username_entry");
	username = g_strdup (gtk_entry_get_text (GTK_ENTRY(w)));
	
	w = glade_xml_get_widget (xml,"auth_password_entry");
	password = g_strdup (gtk_entry_get_text (GTK_ENTRY(w)));

	sussen_sensor_connect (g_host, port, username, password);
}

void set_login_button_state (GtkWidget *widget, gpointer user_data) {

	/* HIG: Behavior of Return key
	 * ref: http://developer.gnome.org/projects/gup/hig/1.0/controls.html#controls-text-return
	 * 
     * - Normally, pressing Return in a dialog should activate the dialog's 
	 *   default button, unless the focused control uses Return for its own 
	 *   purposes. You should therefore set the activates-default property of 
	 *   most entry fields to TRUE.
	 */
		 
	const gchar *widget_text;
	const gchar *user_data_text;
	
	gint ud_text_len = 0;
	
	user_data_text = gtk_entry_get_text (GTK_ENTRY(user_data));
	if (user_data_text) {
		ud_text_len = strlen (user_data_text);
	}	

	widget_text = gtk_entry_get_text (GTK_ENTRY(widget));
	if (widget_text) {
		if (strlen(widget_text) > 0 && ud_text_len > 0) {			
			gtk_widget_set_sensitive (login_button, TRUE);
			gtk_dialog_set_default_response (GTK_DIALOG(auth_dialog), GTK_RESPONSE_OK);
		} else { 
			gtk_widget_set_sensitive (login_button, FALSE);
		}
	}
}

void show_auth_dialog (gchar *host, gchar *port, gchar *user) {
	
	GtkWidget *username_entry;
	GtkWidget *w;
	
	g_host = g_strdup (host);
	g_port = g_strdup (port);
	
	/* 
	 * Create auth dialog dialog
	 */

	xml = glade_xml_new(PACKAGE_DATA_DIR"/sussen.glade", "sussen_auth_dialog", NULL);	
	auth_dialog = glade_xml_get_widget (xml,"sussen_auth_dialog");
	g_signal_connect (GTK_OBJECT (auth_dialog),
					  "key_press_event",
					  G_CALLBACK (on_sussen_auth_dialog_key_press_event),
					  NULL);

	/*
	 * Set dialog label
	 */
	 
	 w = glade_xml_get_widget (xml, "auth_dialog_label");
	 gchar *buf = g_strdup_printf ("Please enter your username and password for '%s'", host);

	 gtk_label_set_text (GTK_LABEL(w), _(buf));
	 g_free (buf);
	
	/*
	 * Set username
	 */
	
	username_entry = glade_xml_get_widget (xml, "auth_username_entry");
	if (user) {
		gtk_entry_set_text (GTK_ENTRY(username_entry), user);
		
		w = glade_xml_get_widget (xml, "auth_password_entry");
		gtk_widget_grab_focus (w);
	} else {
		gtk_widget_grab_focus (username_entry);
	}
	
	/* HIG: Authentication Alerts
	 * ref: http://developer.gnome.org/projects/gup/hig/1.0/windows.html#alerts-authentication
	 *  
	 * - does not enable the OK or equivalent button until all fields have been filled 
	 *   by the user
	 */

	w = glade_xml_get_widget (xml, "auth_password_entry");
	g_signal_connect (GTK_OBJECT (username_entry),
					  "changed",
					  G_CALLBACK (set_login_button_state),
					  (gpointer) w);
	
		
	g_signal_connect (GTK_OBJECT (w),
					  "changed",
					  G_CALLBACK (set_login_button_state),
					  (gpointer) username_entry);
	
	login_button = glade_xml_get_widget (xml,"auth_login_button");
	gtk_widget_set_sensitive (login_button, FALSE);	
	
	/*
	 * Get rid of dialog after user clicks a button
	 */
	
	w = login_button;	
	g_signal_connect (GTK_OBJECT (w),
					  "clicked",
					  G_CALLBACK (do_server_login),
					  NULL);
	
	
	g_signal_connect_swapped (GTK_OBJECT (w),
							  "clicked",
							  G_CALLBACK (gtk_widget_destroy),
							  (gpointer) auth_dialog);
	
	w = glade_xml_get_widget (xml,"auth_cancel_button");
	g_signal_connect_swapped (GTK_OBJECT (w),
							  "clicked",
							  G_CALLBACK (gtk_widget_destroy),
							  (gpointer) auth_dialog);

}

gboolean on_sussen_auth_dialog_key_press_event (GtkWidget* widget, GdkEventKey* event, gpointer data) {

	/* HIG: Behavior of Return key
	 * ref: http://developer.gnome.org/projects/gup/hig/1.0/controls.html#controls-text-return
	 *
	 * - As a further safeguard, remember not to set the default button 
	 *   in a dialog until the minimum amount of required information 
	 *   has been entered, for example, both a username and a password 
	 *	 in a login dialog. Again, in this case you should move focus to
	 *   the next control when the user presses Return, rather than just
	 *   ignoring the keypress.
	 */

	GtkWidget *w;
	
	switch (event->keyval) {

		case GDK_Return:			
			w = glade_xml_get_widget (xml, "auth_password_entry");
			gtk_widget_grab_focus (w);
			break;			
			
		default:
			break;
   	}
	
	return FALSE;
}
