/* 
 * Copyright (C) 2003 Star Chamber Inc.
 *
 * Sussen - sussen-network.c
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 *
 * Author: Loren Bandiera <lorenb@starchamber.ca>
 */

#include "sussen.h"
#include "sussen-backend.h"
#include "sussen-cert-druid.h"
#include "sussen-fileio.h"
#include "sussen-network.h"
#include "sussen-report.h"
#include "sussen-report-druid.h"
#include "sussen-util.h"
#include <stdarg.h>
#include <glib.h>
#include <gnet.h>

GHashTable *scan_hosts;
GHashTable *scanning_current_hosts; 

/* FIXME */
gchar *the_session;
gchar *the_username;
gchar *the_password;
gchar *the_hostname;

static GData* sussen_network_parse_plugins (xmlNodePtr cur) {
	
	gchar *id = xmlGetProp (cur, (const xmlChar *) "id");
	gchar *name = xmlGetProp (cur, (const xmlChar *) "name");
	gchar *category = xmlGetProp (cur, (const xmlChar *) "category");
	gchar *service = xmlGetProp (cur, (const xmlChar *) "service");
	gchar *copyright_year = xmlGetProp (cur, (const xmlChar *) "copyright_year");
	gchar *copyright_name = xmlGetProp (cur, (const xmlChar *) "copyright_name");
	gchar *description = xmlGetProp (cur, (const xmlChar *) "description");
	gchar *summary = xmlGetProp (cur, (const xmlChar *) "summary");
	gchar *family = xmlGetProp (cur, (const xmlChar *) "family");
	gchar *risk = xmlGetProp (cur, (const xmlChar *) "risk");
	gchar *cve_id = xmlGetProp (cur, (const xmlChar *) "cveid");
	gchar *bugtraq_id = xmlGetProp (cur, (const xmlChar *) "bugtraqid");
	gchar *enabled = xmlGetProp (cur, (const xmlChar *) "enabled");	
	
	GData *plugin_datalist;	
	g_datalist_init (&plugin_datalist);

	g_datalist_set_data (&plugin_datalist, "id", g_strdup(id));
	g_datalist_set_data (&plugin_datalist, "category", g_strdup(category));
	g_datalist_set_data (&plugin_datalist, "service", g_strdup(service));
	g_datalist_set_data (&plugin_datalist, "risk", g_strdup(risk));
	g_datalist_set_data (&plugin_datalist, "cve", g_strdup(cve_id));
	g_datalist_set_data (&plugin_datalist, "bugtraq", g_strdup(bugtraq_id));
	g_datalist_set_data (&plugin_datalist, "copyright_year", g_strdup(copyright_year));
	g_datalist_set_data (&plugin_datalist, "copyright_name", g_strdup(copyright_name));
	g_datalist_set_data (&plugin_datalist, "name", g_strdup(name));
	g_datalist_set_data (&plugin_datalist, "family", g_strdup(family));
	g_datalist_set_data (&plugin_datalist, "summary", g_strdup(summary));
	g_datalist_set_data (&plugin_datalist, "description", g_strdup(description));
	g_datalist_set_data (&plugin_datalist, "enabled", g_strdup(enabled));

/*
	gint num_category = atoi (category);
	if ((!num_category == ACT_DENIAL) ||
		(!num_category == ACT_KILL_HOST) ||
		(!num_category == ACT_DESTRUCTIVE_ATTACK)) {			
			g_datalist_set_data (&plugin_datalist, "ENABLED", "0");	
	} else {
		if (!num_category == ACT_SCANNER) {

			if (!g_ascii_strcasecmp (name, "Nmap")) {
				g_datalist_set_data (&plugin_datalist, "ENABLED", "1");	
			} else {
				g_datalist_set_data (&plugin_datalist, "ENABLED", "0");	
			}
		} else {
			g_datalist_set_data (&plugin_datalist, "ENABLED", "1");	
		}		
	}
*/		

	return plugin_datalist;	
}

gboolean update_scan_status (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, gpointer user_data) {
	
	gchar *cur_host;
	GHashTable *hosts = (GHashTable *)user_data;

	if (model && iter) {

		gtk_tree_model_get (GTK_TREE_MODEL(scan_status_store),iter,0,&cur_host,-1);
		if (strcmp(cur_host,g_hash_table_lookup (hosts,"hostname")) == 0) {
			gtk_list_store_set (GTK_LIST_STORE(scan_status_store), iter,
								2, g_strdup(g_hash_table_lookup (hosts,"percent")),
								-1);
		}
		
		g_free (cur_host);
		return FALSE;
	}
	
	return TRUE;
}

void sussen_network_scan_finish (gchar *session_name) {

	set_widgets_sensitive (TRUE,FALSE);
	GtkWidget *w = glade_xml_get_widget (mainwindow_xml, "stop_scan_button");
	gtk_widget_set_sensitive (w,FALSE);
	w = glade_xml_get_widget (mainwindow_xml, "stop_scan1");
	gtk_widget_set_sensitive (w,FALSE);
		
	gint user_response;
	user_response = show_report_druid (g_strdup(session_name));
		
	switch (user_response) {
		
	case REPORT_DRUID_VIEW:
	case REPORT_DRUID_VIEW_EXPORT:
		show_results (g_strdup(session_name));
		break;
			
	case REPORT_DRUID_EXPORT:
		break;
			
	case REPORT_DRUID_CANCEL:
		break;
				
	default:
		break;
	}
}

void parse_cmd (GConn *conn, gchar *cmd) {

	xmlDocPtr doc;
	xmlNodePtr cur;
	
	doc = xmlParseMemory (cmd, strlen(cmd));
	if (!doc) {
		sussen_error_dialog ("Invalid sensor response");		
		return;
	}
	
	cur = xmlDocGetRootElement (doc);
	if (cur) {
		if (g_ascii_strcasecmp(cur->name,"sussen") == 0) {		
	
			gchar *str = g_strdup_printf ("<ssxp version=\"1.0\" />\n");
			gnet_conn_write (conn, str, strlen(str));
			g_free (str);
			return;
		} 

		if (g_ascii_strcasecmp(cur->name,"login") == 0) {

			gchar *str = g_strdup_printf ("<user name=\"%s\" pass=\"%s\" />\n",
											the_username, the_password);
			gnet_conn_write (conn, str, strlen(str));
			g_free (str);
	
			return;
		} 

		if (g_ascii_strcasecmp(cur->name,"authenticated") == 0) {

			gchar *str = g_strdup_printf ("<request data=\"plugins\" />\n");
			gnet_conn_write (conn, str, strlen(str));
			g_free (str);
			
			return;
		}
		
		if (g_ascii_strcasecmp(cur->name,"plugin") == 0) {
			
			GData *plug = sussen_network_parse_plugins (cur);	
			plugin_list = g_slist_append (plugin_list, (gpointer)plug);
			return;
		}

		if (g_ascii_strcasecmp(cur->name,"scan") == 0) {
			gchar *type = xmlGetProp (cur, (const xmlChar *) "type");
						
			if (g_ascii_strcasecmp(type,"start") == 0) {
				gchar *ts = xmlGetProp (cur, (const xmlChar *) "timestamp");
				sussen_backend_update_session_time (ts, 0, the_session);
			} else if (g_ascii_strcasecmp(type,"end") == 0) {
				gchar *ts = xmlGetProp (cur, (const xmlChar *) "timestamp");
				sussen_backend_update_session_time (ts, 1, the_session);
				sussen_network_scan_finish (the_session);
			} else if (g_ascii_strcasecmp(type,"status") == 0) {
				
				gchar *host = xmlGetProp (cur, (const xmlChar *) "host");
				gchar *percent = xmlGetProp (cur, (const xmlChar *) "percent");
			
				g_hash_table_insert (scan_hosts,"hostname",host);
				g_hash_table_insert (scan_hosts,"percent",percent);
				
				gtk_tree_model_foreach (GTK_TREE_MODEL(scan_status_store), update_scan_status,
										(gpointer)scan_hosts);								
			}
			return;
		}

		if (g_ascii_strcasecmp(cur->name,"port") == 0) {

			sussen_backend_insert_port (cmd, the_session);			
			return;
		}

		if (g_ascii_strcasecmp(cur->name,"scandata") == 0) {
		
			sussen_backend_insert_data (cmd, the_session);			
			return;
		}
		
		if (g_ascii_strcasecmp(cur->name,"ready") == 0) {

			gchar *status_msg = g_strdup_printf ("Connected to '%s'", the_hostname);
			update_status (status_msg);
			g_free (status_msg);

			update_connected_status (the_hostname, TRUE);
//			set_target_defaults ();				
			set_widgets_sensitive (TRUE,FALSE);
			
			GtkWidget *w;
	
			w = glade_xml_get_widget (mainwindow_xml,"connect_button");
			gtk_widget_set_sensitive (w, FALSE);
			w = glade_xml_get_widget (mainwindow_xml,"connect1");
			gtk_widget_set_sensitive (w, FALSE);	
			w = glade_xml_get_widget (mainwindow_xml,"stop_scan_button");
			gtk_widget_set_sensitive (w, FALSE);
			w = glade_xml_get_widget (mainwindow_xml,"stop_scan1");
			gtk_widget_set_sensitive (w, FALSE);	
			w = glade_xml_get_widget (mainwindow_xml,"stop_scanning_host_button");
			gtk_widget_set_sensitive (w, FALSE);
			
			return;
		}
		
		if (g_ascii_strcasecmp(cur->name,"error") == 0) {
			
			gchar *msg = xmlGetProp (cur, (const xmlChar *) "msg");
			gchar *str = g_strdup_printf ("sensor error: [%s]\n", msg);
			sussen_error_dialog (str);
			g_free (str);			
			return;
		} 		
	}
}

static void ob_conn_func (GConn* conn, GConnEvent* event, gpointer user_data) {
	
	switch (event->type) {

		case GNET_CONN_CONNECT: {
			gnet_conn_readline (conn);
			break;
		}

		case GNET_CONN_READ: {
			parse_cmd (conn, event->buffer);
			gnet_conn_readline (conn);
			break;
		}

		case GNET_CONN_WRITE: {
			break;
		}

		case GNET_CONN_CLOSE: {
			gnet_conn_delete (conn);
			break;
		}

		case GNET_CONN_TIMEOUT: {
			gnet_conn_delete (conn);
			sussen_error_dialog ("connection timeout\n");
			break;
		}
		
		case GNET_CONN_ERROR: {
			gnet_conn_delete (conn);
			sussen_error_dialog ("communication failure\n");
			break;
		}
	
		default:
			g_assert_not_reached ();
	}	
}

void sussen_sensor_connect (const gchar *hostname, gint port, const gchar *login, const gchar *pass) {
	
	GConn *conn;
	
	conn = gnet_conn_new (hostname, port, ob_conn_func, NULL);
	g_assert (conn);
	
	gnet_conn_connect (conn);
	gnet_conn_set_watch_error (conn, TRUE);	
	
	the_hostname = g_strdup (hostname);
	the_username = g_strdup (login);
	the_password = g_strdup (pass);
	
	global_connection = conn;
}

void sussen_sensor_disconnect (GConn *conn, gchar *hostname) {
	
	gchar *str = g_strdup_printf ("<bye />\n");
	gnet_conn_write (conn, str, strlen(str));
	g_free (str);
	
	gnet_conn_delete (conn);
	global_connection = NULL;
	
	str = g_strdup_printf ("Disconnected from '%s'", hostname);
	update_status (str);	
}

void sussen_sensor_scan (GConn *conn, gchar *target, gchar *session_name) {
	
	// FIXME
	the_session = g_strdup (session_name);
	scan_hosts = g_hash_table_new (g_str_hash, g_str_equal);
		
	gchar *str = g_strdup_printf ("<scan target=\"%s\" policy=\"420\" />\n", target);
	gnet_conn_write (conn, str, strlen(str));
	g_free (str);		
	
	GtkTreeIter iter;
	
	gtk_list_store_append (GTK_LIST_STORE(scan_status_store), &iter);	
	gtk_list_store_set (GTK_LIST_STORE(scan_status_store), &iter,
						0, g_strdup(target),
						-1);
}

// =============================================================================

/*

gchar *attack_plugin_list = NULL;
GHashTable *scan_hosts;
gboolean scan_running;
gboolean restore_session;
GHashTable *scanning_current_hosts; 

void send_server_preference (GQuark key_id, gpointer data, gpointer user_data) {
	
	GTcpSocket *socket = global_gtcpsocket;
	gchar *name;
	gchar *value;

	name = (gchar *)g_quark_to_string (key_id);
	value = (gchar *) data;
	
	gchar *str = g_strdup_printf ("%s <|> %s\n", name, value);
	sussen_network_write (socket, str);
	g_free (str);
}

void build_plugin_list (gpointer data, gpointer user_data) {
	
	GData *cur_plugin = (GData *)data;
	gchar *enabled;
	gchar *id;
	gint value;
		
	id = g_datalist_get_data (&cur_plugin, "ID");
	enabled = g_datalist_get_data (&cur_plugin, "ENABLED");	
	value = atoi (enabled);
	
	if (use_policy_file == 1) {
		gchar *test = g_hash_table_lookup (plugin_hash,id);
		if (test) {
			strcat (attack_plugin_list, id);
			strcat (attack_plugin_list, ";");		
		}		
	} else {	
		if (value == 1) {		
			gchar *id = g_datalist_get_data (&cur_plugin, "ID");
			strcat (attack_plugin_list, id);
			strcat (attack_plugin_list, ";");
		}
	}
}

gint sussen_network_parse_msg (gchar *msg, const gchar *session_name) {

	gint type = 0;
	gchar *sql;
	
	if (g_ascii_strncasecmp (msg, "s:", 2) == 0) {
		return MSG_STAT2;
	} else {
		
		if (g_ascii_strncasecmp (msg, "SERVER", 6) == 0) {

			gchar *cp = g_strdup (msg);
			gchar *srv = g_strstrip(strtok (cp, "<|>"));
			gchar *str_type = g_strstrip(strtok (NULL, "<|>"));
			srv = NULL;
						
			if (!g_ascii_strcasecmp(MSG_HOLE_STR, str_type))
				type = MSG_HOLE;
			
			if (!g_ascii_strcasecmp(MSG_INFO_STR, str_type))
				type = MSG_INFO;
			
			if (!g_ascii_strcasecmp(MSG_NOTE_STR, str_type))
				type = MSG_NOTE;
			
			if (!g_ascii_strcasecmp(MSG_TIME_STR, str_type))
				type = MSG_TIME;

			if (!g_ascii_strcasecmp(MSG_STAT_STR, str_type))
				type = MSG_STAT;

			if (!g_ascii_strcasecmp(MSG_PORT_STR, str_type))
				type = MSG_PORT;

			if (!g_ascii_strcasecmp(MSG_ERROR_STR, str_type))
				type = MSG_ERROR;

			if (!g_ascii_strcasecmp(MSG_PING_STR, str_type))
				type = MSG_PING;

			if (!g_ascii_strcasecmp(MSG_PLUGINS_ORDER_STR, str_type))
				type = MSG_PLUGINS_ORDER;

			if (!g_ascii_strcasecmp(MSG_FINISHED_STR, str_type))
				type = MSG_FINISHED;
			
			if (!g_ascii_strcasecmp(MSG_BYE_STR, str_type))
				type = MSG_BYE;
			
			switch (type) {
  
				case MSG_TIME: {
					
					gint type = 0;
					gchar *new_msg = g_strstrip(strtok (NULL, "<|>"));

					if (g_ascii_strcasecmp (new_msg, "SCAN_START") == 0){
						type = TIMER_SCAN_START;
					} else if(g_ascii_strcasecmp (new_msg, "SCAN_END") == 0) {
						type = TIMER_SCAN_END;
					}

					if (type) {
*/
						/* FIXME: need to reformat date to YYYY-MM-DD HH:MM:SS */						
/*						gchar *date = g_strstrip(strtok (NULL, "<|>"));
						
						switch (type) {
							
							case TIMER_SCAN_START: {

								sql = g_strdup_printf ("UPDATE sessions SET start_time='%s' WHERE session_id='%s'", date, session_name);
								sussen_backend_non_query (sussen_be_connection, sql);								
							  	g_free (sql);								
								break;
							}
							
							case TIMER_SCAN_END: {

								sql = g_strdup_printf ("UPDATE sessions SET end_time='%s' WHERE session_id='%s'", date, session_name);
								sussen_backend_non_query (sussen_be_connection, sql);								
							  	g_free (sql);
								break;
							}
							
						}
					}
					break;
				}
				
				case MSG_STAT2: {
					return (MSG_STAT2);
					break;
				}

				case MSG_ERROR: {
					break;
				}
				
				case MSG_PORT: {
					sussen_backend_insert_port (msg, session_name);
					break;
				}

				case MSG_HOLE: {
					sussen_backend_insert_data (msg, MSG_HOLE, session_name);
					break;
				}

				case MSG_INFO: {
					sussen_backend_insert_data (msg, MSG_INFO, session_name);
					break;
				}

				case MSG_NOTE: {
					sussen_backend_insert_data (msg, MSG_NOTE, session_name);					
					break;
				}

				case MSG_STAT: {
					return(MSG_STAT);
					break;
				}

				case MSG_FINISHED: {				
					break;
				}
				
				case MSG_PING: {
					
					gchar *str = g_strdup_printf ("CLIENT <|> PONG <|> PING <|> CLIENT\n\n");
					sussen_network_write (global_gtcpsocket, str);
					g_free (str);
					return MSG_PING;
					break;
				}

				case MSG_PLUGINS_ORDER: {
					return (MSG_PLUGINS_ORDER);
					break;
				}

				case MSG_BYE: {
					return (MSG_BYE);
					break;
				}				
			}
			
			return -1;
			
		} else {
			return -1;
		}
	}
}

gboolean update_each_attack (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, GHashTable *hosts) {
	
	gchar *cur_host;
		
	if (model && iter) {

		gtk_tree_model_get (GTK_TREE_MODEL(scan_status_store),iter,0,&cur_host,-1);
		if (strcmp(cur_host,g_hash_table_lookup (hosts,"hostname")) == 0) {
			gtk_list_store_set (GTK_LIST_STORE(scan_status_store), iter,
								2, g_strdup(g_hash_table_lookup (hosts,"percent")),
								-1);
		} 
		
		g_free (cur_host);
		return FALSE;
	}
	
	return TRUE;
}


gboolean update_each_portscan (GtkTreeModel *model, GtkTreePath *path, GtkTreeIter *iter, GHashTable *hosts) {

	gchar *cur_host;
		
	if (model && iter) {

		gtk_tree_model_get (GTK_TREE_MODEL(scan_status_store),iter,0,&cur_host,-1);
		if (strcmp(cur_host,g_hash_table_lookup (hosts,"hostname")) == 0) {
			gtk_list_store_set (GTK_LIST_STORE(scan_status_store), iter,
									1, g_strdup(g_hash_table_lookup (hosts,"percent")),
									-1);
		} 
		
		g_free (cur_host);
		return FALSE;
	}
	
	return TRUE;
}

void update_scan_status (gchar *msg, gint short_status) {

	gchar *hostname = NULL;
	gchar *action = NULL;
	gchar *current = NULL;
	gint max = 0;
	gfloat gmax;
	gfloat gcurrent;
	
	if (!short_status) {
		g_print ("msg == [%s]\n", msg);
	} else {
		gchar *cp = g_strstrip(g_strdup (msg));
		strtok (cp, ":");
		gchar *check_action = strtok (NULL, ":");
		if (g_ascii_strncasecmp (check_action, "p", 1) == 0) {
			action = g_strdup ("portscan");
		} else {
			action = g_strdup ("attack");
		}
		
		hostname = strtok (NULL, ":");
		current = strtok (NULL, ":");
		max = atoi (strtok (NULL, ":"));
	}
	
	if (hostname) {

		if (g_hash_table_lookup (scanning_current_hosts, (gconstpointer) hostname) != NULL) {
		
			gfloat f;

			gmax = max;
			gcurrent = atoi(current);
			f = (gcurrent/gmax);

			if(f>1)
				f=1;
			if(f<0.0)
				f=0.0;		
		
			gint test;
			test = f * 100;
			gchar *test2 = g_strdup_printf ("%d%%", test);
			g_hash_table_insert (scan_hosts,"hostname",hostname);
			g_hash_table_insert (scan_hosts,"percent", test2);
			
			if( strcmp(action, "portscan") == 0 ) {
				gtk_tree_model_foreach (GTK_TREE_MODEL(scan_status_store), (void *)update_each_portscan,
										scan_hosts);
			} else {
				gtk_tree_model_foreach (GTK_TREE_MODEL(scan_status_store), (void *)update_each_attack,
										scan_hosts);
			}
			g_free (test2);				
		} else {

			g_hash_table_insert (scanning_current_hosts, hostname, "yes");
			
			GtkTreeIter iter;
			
			gtk_list_store_append (scan_status_store,&iter);
			gtk_list_store_set (GTK_LIST_STORE(scan_status_store), &iter,
								0, g_strdup(hostname),
								-1);
			
		}
		
	}
}

gboolean io_read_callback (GIOChannel *source, GIOCondition condition, gpointer data) {
	
	gchar *buf;
	gboolean done = FALSE;
	gint type = 0;
	gint finished = 0;
	gchar *session_name = (gchar*)data;
	
	if (!global_gtcpsocket) {
		sussen_error_dialog ("connection to server lost\n");
		done = TRUE;
	}

	while (!done) {
		
		buf = sussen_network_read (global_gtcpsocket, 32678);
		if (!buf) {
			done = TRUE;
		} else {
			type = sussen_network_parse_msg (g_strstrip(buf), g_strdup(session_name));

			switch(type) {
				case MSG_BYE: {
					gchar *str = g_strdup_printf ("CLIENT <|> BYE <|> ACK\n");
					sussen_network_write (global_gtcpsocket, str);
					g_free (str);
					finished = 1;
					break;
				}
		
				case MSG_STAT2 : 
					update_scan_status (buf, 1);
					break;
	
				case MSG_STAT :
					update_scan_status (buf, 0);
					break;

				case MSG_PLUGINS_ORDER :
*/				
					/* FIXME: haven't yet gotten this message from the server; not sure
					   what to do with it */
/*					break;

				case MSG_FINISHED :				
					break;	
			}
			
			if (finished) {				
				scan_running = FALSE;

				if (restore_session)
					restore_session = FALSE;
		
				set_widgets_sensitive (TRUE,FALSE);
				GtkWidget *w = glade_xml_get_widget (mainwindow_xml, "stop_scan_button");
				gtk_widget_set_sensitive (w,FALSE);
				w = glade_xml_get_widget (mainwindow_xml, "stop_scan1");
				gtk_widget_set_sensitive (w,FALSE);
		
				gint user_response;
				user_response = show_report_druid (g_strdup(session_name));
		
				switch (user_response) {
			
					case REPORT_DRUID_VIEW:
					case REPORT_DRUID_VIEW_EXPORT:
						show_results (g_strdup(session_name));
						break;
			
					case REPORT_DRUID_EXPORT:
						break;
			
					case REPORT_DRUID_CANCEL:
						break;
				
					default:
						break;
				}			
			} 
			
			g_free (buf);
			return TRUE;
		}
	}
	
	return FALSE;
}

void sussen_network_send_preferences (void) {
	
	GTcpSocket *socket = global_gtcpsocket;
	gchar *str;
	gchar *buf;

	attack_plugin_list = g_malloc (2000*50+1+5*50+1);
	g_slist_foreach (plugin_list, build_plugin_list, NULL);
*/		
	/* FIXME: manually adding two scanners to the plugin_list */
//	strcat (attack_plugin_list, "10180;10335;");
		
//	scanning_current_hosts = g_hash_table_new (g_str_hash, g_str_equal);
//	scan_hosts = g_hash_table_new (g_str_hash, g_str_equal);
	
	/* send preferences */
/*	
	str = g_strdup_printf ("CLIENT <|> PREFERENCES <|>\n");
	sussen_network_write (socket, str);
	g_free (str);
	
	str = g_strdup_printf ("ntp_opt_show_end <|> yes\n");
	sussen_network_write (socket, str);
	g_free (str);

	str = g_strdup_printf ("ntp_keep_communication_alive <|> yes\n");
	sussen_network_write (socket, str);
	g_free (str);

	str = g_strdup_printf ("ntp_short_status <|> yes\n");
	sussen_network_write (socket, str);
	g_free (str);

	str = g_strdup_printf ("ntp_client_accepts_notes <|> yes\n");
	sussen_network_write (socket, str);
	g_free (str);

	str = g_strdup_printf ("ntp_escape_crlf <|> yes\n");
	sussen_network_write (socket, str);
	g_free (str);

	g_datalist_set_data (&server_preferences, "plugin_set", g_strdup(attack_plugin_list));
	g_datalist_foreach (&server_preferences, send_server_preference, NULL);	
*/	
	/* FIXME: send plugin preferences 
	 *
	 * example
	 *	%s[%s]:%s <|> %s\n
	 */
/*
	str = g_strdup_printf ("<|> CLIENT\n");
	sussen_network_write (socket, str);
	g_free (str);
*/	
	/* get_preferences_errors */
/*	buf = sussen_network_read (socket, 512);
	g_free (buf);
	
	buf = sussen_network_read (socket, 512);
	g_free (buf);	
}

void sussen_network_attack (const gchar *hostname, const gchar *session_name) {

	GTcpSocket *socket = global_gtcpsocket;
	gchar *str;

	sussen_network_send_preferences ();	
*/	
	/* FIXME: set rules */
	
	/* Send attack command */
/*	
	str = g_strdup_printf ("CLIENT <|> LONG_ATTACK <|>\n");
	sussen_network_write (socket, str);
	g_free (str);

	str = g_strdup_printf ("%d\n", strlen(hostname));
	sussen_network_write (socket, str);
	g_free (str);

	str = g_strdup_printf ("%s\n", hostname);
	sussen_network_write (socket, str);
	g_free (str);
	
	GIOChannel *gio = gnet_tcp_socket_get_io_channel (socket);
	g_io_add_watch (gio, G_IO_IN, io_read_callback, (gpointer)session_name);

	g_free (attack_plugin_list);
}

void sussen_network_restore_session (const gchar *session_name) {
	
	GTcpSocket *socket = global_gtcpsocket;
	gchar *str;

	sussen_network_send_preferences ();	
*/	
	/* FIXME: set rules */
/*
	str = g_strdup_printf ("CLIENT <|> SESSION_RESTORE <|> %s <|> CLIENT\n", session_name);
	sussen_network_write (socket, str);
	g_free (str);
	
	GIOChannel *gio = gnet_tcp_socket_get_io_channel (socket);
	g_io_add_watch (gio, G_IO_IN, io_read_callback, (gpointer)session_name);
	
	g_free (attack_plugin_list);
}
*/
